/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.request;

import org.netkernel.urii.*;

/** Each request has an associated scope within which it is evaluated.
 * This scope is implemented as a linked list of IRequestScopeLevel objects.
 * <p>
 * Each IRequestScopeLevel refers to an ISpace object and its
 * parent IRequestScopeLevel object.
 * <p>
 *
 * @author  tab
 */
public interface IRequestScopeLevel
{
	/** Returns the request scope level that is the immediate parent of this scope level.
   *
   * If this scope level is the outermost level (where getDepth() returns +1),
   * this method returns null.
   *
	 * @return The parent scope level.
	 */
	IRequestScopeLevel getParent();
	
	/** Returns the ISpace at this level in the request scope.
   *
   * @return The space at this scope level.
	 */
	ISpace getSpace();
	
	/** The absolute depth of this scope level from the outermost parent level.
   *
   * The first, most outer scope level has a depth of one (1).
   *
	 * @return The depth of this scope level relative to the outermost parent scope level.
	 */
	int getDepth();
	
	/** The available depth of the scope level before we hit a null parent.
	 *  A scope with no parent will have an available depth of one (1).
	 * @return The depth of this scope level relative to a null parent
	 */
	int getAvailableDepth();
	
	/** Returns an indication if the space at this scope level is durable.
   *
   * A durable scope level is not lost when scope levels are popped
   * during request processing.
   * <p>
   * 
	 * @return Indicator if this scope level refers to a "durable space".
	 */
	boolean isDurable();
}