/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.scheduler;

import java.io.IOException;
import java.io.Writer;
import org.netkernel.request.IRequest;
import org.netkernel.request.impl.RequestFactory;
import org.netkernel.urii.impl.NetKernelError;
import org.netkernel.util.XMLUtils;

/**
 * NetKernel exception is a generic exception class used for most exceptions in the NetKernel. It is
 * nestable and allows multiple exceptions to be nested at any depth. Its most important characteristic
 * is that its state can be recursively dumped as XML giving a clear picture of an exception scenario is
 * a document.
 * @author  tab
 */
public class RequestFrameError extends NetKernelError
{
	private static final long serialVersionUID = 6109578877391438584L;
	
	/** request that caused exception */
	private final IRequest mRequest;
	
	/**
	 * Constructs an instance of <code>RequestFrameError</code> 
	 * @param aRequestIn
	 */
	public RequestFrameError(IRequest aRequestIn, Throwable aCause)
	{	super(aCause);
		mRequest = aRequestIn;
	}
	
	/** @Return the id for the exception
	 */
	public String getId()
	{	return "RequestFrameError";
	}
	
	/* @return the a request id that caused the error
	 * @see org.netkernel.util.IException#getRequestId()
	 */
	public IRequest getRequestIn()
	{	return mRequest;
	}
	
	public void writeAdditionalFieldsAsXML(Writer aWriter, int aIndent) throws IOException
	{	XMLUtils.writeElement(aWriter, "request",XMLUtils.escape(RequestFactory.toShortString(mRequest)),aIndent);
	}
}
