/******************************************************************************
  (c) Copyright 2002-2006, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile$
  Version:       $Name$ $Revision$
  Last Modified: $Date$
 *****************************************************************************/
/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/package org.netkernel.scheduler;

 
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.netkernel.container.IKernel;
import org.netkernel.container.ILogger;
import org.netkernel.container.ITimer;
import org.netkernel.container.config.IConfiguration;
import org.netkernel.container.config.IConfigurationListener;
import org.netkernel.request.*;
import org.netkernel.request.impl.*;
import org.netkernel.urii.INetKernelException;
import org.netkernel.urii.impl.NetKernelError;
import org.netkernel.urii.impl.NetKernelException;
import org.netkernel.util.*;

public class Scheduler implements IConfigurationListener
{
	private final IKernel mKernel;
	private final PriorityRequestTable mRequestTable;
	private WorkerThreadPool mThreadPool;
	private int mMaxStackDepth;
	private LockedEndpointTable mLockedEndpoints;
	private int mSetThreadCount;
	private ITimer mTimer;
	private final GroundState mGroundState = new GroundState();
	private int mCostAccumulationThreshold;
	
	public Scheduler(IKernel aKernel)
	{	mKernel=aKernel;
		mKernel.addConfigurationListener(this);
		mRequestTable = new PriorityRequestTable();
	}
	
	public void start(ThreadGroup aRootThreadGroup)
	{	
		mLockedEndpoints = new LockedEndpointTable();
		innerLoadConfiguration();
		
		if (mThreadPool==null)
		{	mThreadPool=new WorkerThreadPool(mKernel.getLogger().format("SCHEDULER_THREADPOOL"), mSetThreadCount, aRootThreadGroup, this );
		}
		else
		{	mThreadPool.setCount(mSetThreadCount);
		}
		mThreadPool.start();
	}
	
	public void configurationChanged(IConfiguration aConfiguration)
	{	if (!isStopped())
		{	innerLoadConfiguration();
		}
	}
	
	private void innerLoadConfiguration()
	{	IConfiguration c=mKernel.getConfiguration();
		mMaxStackDepth = c.getInt("netkernel.scheduler.maxstackdepth",32);
		int lastSetThreadCount = mSetThreadCount;
		mSetThreadCount = c.getInt("netkernel.scheduler.threadcount",4);
		if (mThreadPool!=null && lastSetThreadCount!=mSetThreadCount)
		{	mThreadPool.setCount(mSetThreadCount);
			mRequestTable.interrupt();
		}
		mTimer=mKernel.getTimer();
		
		float corePerformance=64.0f;
		try
		{	corePerformance = NumberFormat.getInstance().parse(c.getString("netkernel.corePerformance")).floatValue();
		}
		catch(Exception e){;}
		mCostAccumulationThreshold=Math.round((0.5f*(float)mTimer.getRelativeTimeDivisor())/corePerformance);
	}
	
	public boolean hasPendingRequests()
	{	return mGroundState.countRootRequests()>0;
	}
	
	public void stop(boolean aForce)
	{	
		if (!aForce)
		{
			List<RootState> roots=getRootRequestStates();
			if (!roots.isEmpty())
			{	String killExId=getKernel().getLogger().format("EX_KILL_ON_STOP");
				for (RootState root: roots)
				{	kill(root.getId(), killExId);
				}
			}
			mRequestTable.interrupt();
		}
		
		mThreadPool.stop(aForce);
	}
	
	public boolean isStopped()
	{	return (mThreadPool==null || mThreadPool.threadCount()==0);
	}
	
	public IKernel getKernel()
	{	return mKernel;
	}
	
	PriorityRequestTable getRequestTable()
	{	return mRequestTable;
	}
	
	LockedEndpointTable getLockedEndpoints()
	{	return mLockedEndpoints;
	}
	
	final long getRelativeTime()
	{	return mTimer.getRelativeTime();
	}

	final long getAbsoluteTime()
	{	return mTimer.getAbsoluteTime();
	}
	
	final long getApproximateTime()
	{	return mTimer.getApproximateTime();
	}
	
	final int getMaxStackDepth()
	{	return mMaxStackDepth;
	}
	
	final int getCostAccumulationThreshold()
	{	return mCostAccumulationThreshold;
	}
	
	public void onAsyncRequest(IRequest aRequest)
	{	RequestState parentState=getParentState(aRequest);
		RequestState newState;
		if (parentState!=null)
		{	newState = new RequestState(parentState,aRequest);
			parentState.declareSubrequestIssue(newState);
			INetKernelException sdep = isStackDepthExceeded(parentState);
			if (sdep!=null)
			{	newState.setExceptionResponse(sdep,MetaImpl.DEPENDENT_EXPIRY);
			}
		}
		else
		{	newState = new RootState(this,aRequest,aRequest);
			mGroundState.declareSubrequestIssue(newState);
		}
		newState.declareAsync();
		mRequestTable.put(newState);
	}
	
	public IResponse synchronousRequest(IRequest aRequest) throws INetKernelException 
	{	SynchronousRequestee sr = new SynchronousRequestee();
		IRequest request = RequestFactory.cloneWithRequestor(aRequest,sr);
		
		RequestState parentState=getParentState(aRequest);
		RequestState newState;
		if (parentState!=null)
		{	INetKernelException sdep=isStackDepthExceeded(parentState);
			if (sdep!=null)
			{	throw sdep;
			}
			parentState.pauseTimer();
			newState = new RequestState(parentState,request);
			
			parentState.declareSubrequestIssue(newState);
		}
		else
		{	newState = new RootState(this,request,aRequest);
			mGroundState.declareSubrequestIssue(newState);
		}		
		
		Thread currentThread=Thread.currentThread();
		ClassLoader contextClassLoader=currentThread.getContextClassLoader();
		newState.processRequest();
		IResponse result=null;
		do
		{	try
			{	result=sr.getResponse();
				if (result==null)
				{	result=sr.waitForResponse();
				}
			}
			catch (InterruptedException e)
			{	// silently ignore, we need a response
			}
			catch (IllegalStateException e)
			{	// thrown when we have a null response - should never happen
				Exception ex=new Exception("Null response in Scheduler.synchronousRequest");
				result=new ResponseImpl(request, ex, MetaImpl.EXPIRED);
			}
		}
		while (result==null);
		
		currentThread.setContextClassLoader(contextClassLoader);
		if (parentState!=null)
		{	parentState.resumeTimer();
		}

		Object representation=result.getRepresentation();
		if (representation instanceof INetKernelException)
		{	throw (INetKernelException)representation;
		}
		else if (representation instanceof NetKernelError)
		{	throw (NetKernelError)representation;
		}
		else
		{	result = new ResponseImpl(aRequest,result.getRepresentation(),result.getMeta());
		}
		return result;
	}
	
	private INetKernelException isStackDepthExceeded(IRequestState aState)
	{	IRequestState rs=aState;
		int depth=0;
		do
		{	depth++;
			rs=rs.getParent();
		} while (rs!=null);
		
		if (depth>mMaxStackDepth)
		{	return Utils.createFormattedException("EX_STACK_DEPTH", "MSG_STACK_DEPTH", mKernel.getLogger(),mMaxStackDepth);
		}
		else
		{	return null;
		}
	}
	
	RequestState getParentState(IRequest aRequest)
	{	IRequest parent = aRequest.getParent();
		RequestState result=null;
		while (parent!=null)
		{	IRequestor requestor = parent.getRequestor();
			if (requestor instanceof RequestState)
			{	result = (RequestState)requestor;
				break;
			}
			else
			{	parent=parent.getParent();
			}
		}
		return result;
	}
	
	GroundState getGroundState()
	{	return mGroundState;
	}
	
	public MultiMap dumpBusyEndpoints()
	{	return mLockedEndpoints.dump();
	}
	
	public RequestState[] dumpRequestTable()
	{	return mRequestTable.dump();
	}
	
	public List<WorkerThread> getActiveThreads()
	{	return mThreadPool.getActiveThreads();
	}
	
	public long getLastStarved()
	{	return mThreadPool.getLastStarved();
	}
	
	public List<RootState> getRootRequestStates()
	{	return mGroundState.getRoots();
	}
	
	public boolean kill(long aRootId, String aKillExId)
	{
		boolean result=false;
		
		RootState root=getRootState(aRootId);
		if (root!=null)
		{	
			getKernel().getLogger().log(ILogger.WARNING, this, "MSG_KILL_ROOT", aRootId, RequestFactory.toShortString(root.getRequestIn()),aKillExId);
			
			//find queued states
			List<RequestState> queued = clearQueue(root);
			for (RequestState rs: queued)
			{	rs.kill(aKillExId);
			}
			
			//find locked-on-endpoint states
			List<RequestState> locked = clearLockedEndpoints(root);
			for (RequestState rs: locked)
			{	rs.kill(aKillExId);
			}
			
			//find responseless states
			List<RequestState> responseless=getResponselessEndpoints(root);
			for (RequestState rs: responseless)
			{	rs.kill(aKillExId);
			}
			
			//pause for fallout
			if (queued.size()>0 || locked.size()>0 || responseless.size()>0)
			{	try
				{	Thread.sleep(100);
				}
				catch (Exception e) {;}
			}
			
			Set<Thread> threads=new HashSet<Thread>();
			recurseStateForThreads(root, threads);
		
			//kill threads
			//NetKernelError error = new NetKernelError(new NetKernelException(aKillExId));
			for (Thread t: threads)
			{	
				StackTraceElement[] stes = t.getStackTrace();
				if (stes.length>4 && stes[4].getClassName().endsWith("PriorityRequestTable"))
				{	//idle
					continue;
				}
				getKernel().getLogger().log(ILogger.WARNING, this, "MSG_KILL_THREAD", t.getName(), aKillExId);
				//t.stop(error);
				t.stop();
			}
			
			if (threads.size()>0)
			{	try
				{	Thread.sleep(100);
				}
				catch (Exception e) {;}
			}
			
			root=getRootState(aRootId);
			result=(root==null);
			
		}
		else
		{	result=true;
		}
		
		return result;
	}
	
	private void recurseStateForThreads(RequestState aState, Set<Thread> aThreads)
	{	Thread t=aState.getExecutionThread();
		if (t!=null)
		{	aThreads.add(t);
		}
		List<RequestState> substates=aState.getSubStates();
		for (RequestState substate: substates)
		{	recurseStateForThreads(substate,aThreads);
		}
	}
	
	private RootState getRootState(long aId)
	{	RootState result=null;
		List<RootState> rootStates = getRootRequestStates();
		RootState root=null;
		for (RootState rs: rootStates)
		{	if (rs.getId()==aId)
			{	result=rs;
				break;
			}
		}
		return result;
	}
	
	
	private List<RequestState> clearQueue(RootState aRoot)
	{	return mRequestTable.clear(aRoot);
	}
	
	private List<RequestState> clearLockedEndpoints(RootState aRoot)
	{	return mLockedEndpoints.clear(aRoot);
	}
	
	private List<RequestState> getResponselessEndpoints(RootState aRoot)
	{
		List<RequestState> result=new ArrayList<RequestState>();
		recurseGetResponselessEndpoints(aRoot,result);
		
		Map<Long,RequestState> lastStateOnThread = new HashMap<Long,RequestState>();
		for (RequestState root: getRootRequestStates())
		{	recurseStateForThreads(root,lastStateOnThread);
		}
		
		
		
		return result;
	}
	
	private void recurseStateForThreads(RequestState aState, Map<Long,RequestState> aLastStateOnThread)
	{	Thread t=aState.getExecutionThread();
		if (t!=null)
		{	Long id=t.getId();
			RequestState latest=aLastStateOnThread.get(id);
			if (latest!=null)
			{	if (latest.getTimestamp()<=aState.getTimestamp())
				{	aLastStateOnThread.put(id, aState);
				}
			}
			else
			{	aLastStateOnThread.put(id, aState);
			}
		}
		List<RequestState> substates=aState.getSubStates();
		for (RequestState substate: substates)
		{	recurseStateForThreads(substate,aLastStateOnThread);
		}
	}	
	
	private void recurseGetResponselessEndpoints(RequestState aState, List<RequestState> aResponseless)
	{	if (aState.isResponseless())
		{	aResponseless.add(aState);
		}
		for (RequestState state: aState.getSubStates())
		{	recurseGetResponselessEndpoints(state,aResponseless);
		}
	}
}