/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.util;

import java.io.*;

/** Basic XML formatting utilities
 * @author Tony Butterfield
 */
public class XMLUtils
{
	/** escape the given string into a form safe to embed into an XML stream
	 */
	public static String escape(String aXML)
	{	int len = aXML.length();
		StringBuilder result = new StringBuilder(len*2);
		for (int i=0; i<len; i++)
		{	char c = aXML.charAt(i);
			switch (c)
			{	case '<':
					result.append("&lt;");
					break;
				case '>':
					result.append("&gt;");
					break;
				case '\'':
					result.append("&apos;");
					break;
				case '"':
					result.append("&quot;");
					break;
				case '&':
					result.append("&amp;");
					break;
				default:
					if (c>0xFFFD || (c<0x0020 && c!='\t' && c!='\n' && c!='\r'))
					{	result.append("&x");
						result.append(Integer.toHexString(c));
						result.append(';');
					}
					else
					{	result.append(c);
					}
			}
		}
		return result.toString();
	}
	
	
	public static void writeOpenTag(Writer osw, String aName, int aDepth) throws IOException
	{	boolean tidy=(aDepth>=0);
		if (tidy) writeIndent(osw,aDepth);
		osw.write('<');
		osw.write(aName);
		osw.write('>');	
		if (tidy) osw.write('\n');
	}
	public static void writeCloseTag(Writer osw, String aName, int aDepth) throws IOException
	{	boolean tidy=(aDepth>0);
		if (tidy) writeIndent(osw,aDepth);
		osw.write("</");
		osw.write(aName);
		osw.write('>');	
		if (tidy) osw.write('\n');
	}
	
	/** Write an XML element with text to the writer
	 * @param osw writer to write to
	 * @param aName name of the element
	 * @param aValue an escaped text value to put between the tags
	 * @param aDepth an indent depth for pretty printing
	 * @throws IOException thrown on failure to write to writer
	 */
	public static void writeElement(Writer osw, String aName, String aValue, int aDepth) throws IOException
	{	boolean tidy=(aDepth>=0);
		if (tidy) writeIndent(osw,aDepth);
		writeOpenTag(osw,aName,-1);
		osw.write(aValue);
		writeCloseTag(osw,aName,-1);
		if (tidy) osw.write('\n');
	}
	
	public static void writeIndent(Writer osw, int aDepth) throws IOException
	{	for (int i=0; i<aDepth; i++)
		{	osw.write(' ');
		}
	}
}
